import {
  readInSchemaFromBackend,
  useAuth,
  useAuthorizationCache,
  useDatabaseAPI,
  useQueryAPI,
  useQuerybuilderGraph,
  useQuerybuilderHash,
  useSchemaAPI,
  useSessionCache
} from '@graphpolaris/shared/lib/data-access';
import { WebSocketHandler } from '@graphpolaris/shared/lib/data-access/socket';
import Broker from '@graphpolaris/shared/lib/data-access/socket/broker';
import {
  assignNewGraphQueryResult,
  useAppDispatch,
} from '@graphpolaris/shared/lib/data-access/store';
import { GraphQueryResultFromBackend, resetGraphQueryResults } from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';
import {
  Query2BackendQuery,
  QueryBuilder,
} from '@graphpolaris/shared/lib/querybuilder';
import { Schema } from '@graphpolaris/shared/lib/schema/panel';
import { useEffect, useRef, useState } from 'react';
import { Navbar } from '../components/navbar/navbar';
import Panel from '../components/panels/panel';
import { domain } from '../environments/variables';
import { VisualizationPanel } from './panels/Visualization';
import styles from './app.module.scss';
import { logout } from '@graphpolaris/shared/lib/data-access/store/authSlice';
import { SchemaFromBackend } from '@graphpolaris/shared/lib/schema';

export interface App {
  styles: any;
}

export function App(props: App) {
  const isLogin = useAuth();
  const auth = useAuthorizationCache()
  const api = useDatabaseAPI(domain);
  const api_schema = useSchemaAPI(domain);
  const api_query = useQueryAPI(domain);
  const dispatch = useAppDispatch();
  const session = useSessionCache();
  const query = useQuerybuilderGraph();
  const queryHash = useQuerybuilderHash();
  const ws = useRef(new WebSocketHandler(domain));
  const [authCheck, setAuthCheck] = useState(false);

  // for testing purposes
  // useEffect(() => {
  //   console.info('Authentification changed', auth)
  // }, [auth]);

  useEffect(() => {
    // Default
    Broker.instance().subscribe(
      (data: SchemaFromBackend) => dispatch(readInSchemaFromBackend(data)),
      'schema_result'
    );

    Broker.instance().subscribe(
      (data: GraphQueryResultFromBackend) =>
        dispatch(assignNewGraphQueryResult(data)),
      'query_result'
    );

    return () => {
      Broker.instance().unSubscribeAll('schema_result');
      Broker.instance().unSubscribeAll('query_result');
    };
  }, []);

  useEffect(() => {
    // New active database
    if (auth.accessToken && session.currentDatabase) {
      ws.current.useToken(auth.accessToken).connect(() => {
        api_schema.RequestSchema(session.currentDatabase);
      });
    }
  }, [session.currentDatabase]);

  useEffect(() => {
    // Newly (un)authorized
    if (auth.authorized) {
      console.info("App is authorized; Getting Datatabases", isLogin);
      api.GetAllDatabases({ updateSessionCache: true });
      setAuthCheck(true);
    } else {
      dispatch(logout())
    }
  }, [isLogin]);

  useEffect(() => {
    // New query
    if (session?.currentDatabase && query) {
      if (query.edges.length === 0) {
        dispatch(resetGraphQueryResults());
      } else
        api_query.execute(Query2BackendQuery(session.currentDatabase, query));
    }
  }, [queryHash]);

  return (
    <div className="h-screen w-screen">
      <div
        className={
          'flex h-screen w-screen overflow-hidden ' +
          (!auth.authorized ? 'blur-sm pointer-events-none ' : '')
        }
      >
        <div className="h-full relative flex flex-col flex-1 overflow-y-auto overflow-x-hidden">
          <div className="h-fit flex-grow-0">
            <Navbar />
          </div>
          <main className={styles.mainContainer}>
            <div className={styles.schema}>
              <Panel content="Schema">
                <Schema auth={authCheck} />
              </Panel>
            </div>
            <div className={styles.panel}>
              <div className={styles.visualization}>
                <VisualizationPanel />
              </div>
              <div className={styles.queryBuilder}>
                <Panel content="Query Panel">
                  <QueryBuilder />
                </Panel>
              </div>
            </div>
          </main>
        </div>
      </div>
      {/*<GridLayout*/}
      {/*  className="layout"*/}
      {/*  cols={10}*/}
      {/*  rowHeight={30}*/}
      {/*  width={window.innerWidth}*/}
      {/*>*/}
      {/*  <div*/}
      {/*    key="schema-panel"*/}
      {/*    data-grid={{ x: 0, y: 0, w: 3, h: 30, maxW: 5, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <Panel content="Schema Panel" color="red">*/}
      {/*      <Schema />*/}
      {/*    </Panel>*/}
      {/*  </div>*/}
      {/*  <div*/}
      {/*    key="query-panel"*/}
      {/*    data-grid={{ x: 3, y: 20, w: 5, h: 10, maxH: 20, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <Panel content="Query Panel" color="blue">*/}
      {/*      <QueryBuilder />*/}
      {/*    </Panel>*/}
      {/*  </div>*/}
      {/*  <div*/}
      {/*    key="visualisation-panel"*/}
      {/*    data-grid={{ x: 3, y: 0, w: 7, h: 20, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <VisualizationPanel />*/}
      {/*  </div>*/}
      {/*  <div*/}
      {/*    key="history-panel"*/}
      {/*    data-grid={{ x: 8, y: 20, w: 2, h: 10, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <Panel content="History Channel" color="purple"></Panel>*/}
      {/*  </div>*/}
      {/*</GridLayout>*/}
    </div>
  );
}

export default App;
