/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import { select } from 'd3';
import React, { useEffect, useRef } from 'react';
// import style from './PaohvisComponent.module.scss';
import { HyperEdgeI } from '../Types';

type HyperEdgeRangeProps = {
  index: number;
  text: string;
  hyperEdges: HyperEdgeI[];
  nRows: number;
  colOffset: number;
  xOffset: number;
  yOffset: number;
  rowHeight: number;
  hyperedgeColumnWidth: number;
  gapBetweenRanges: number;

  onMouseEnter: (col: number, nameToShow: string) => void;
  onMouseLeave: (col: number) => void;
};
export const HyperEdgeRange = (props: HyperEdgeRangeProps) => {
  // calculate the width of the hyperEdgeRange.
  const rectWidth = props.hyperEdges.length * props.hyperedgeColumnWidth + 2 * props.gapBetweenRanges;

  const rows: JSX.Element[] = [];
  for (let i = 0; i < props.nRows; i++)
    rows.push(
      <g key={i} className={'row-' + i}>
        <rect
          key={i.toString()}
          y={i * props.rowHeight}
          width={rectWidth}
          height={props.rowHeight}
          fill={i % 2 === 0 ? '#e6e6e6' : '#f5f5f5'}
          style={{ zIndex: -1 }}
        />
      </g>,
    );

  let hyperedges = props.hyperEdges.map((hyperEdge, i) => (
    <HyperEdge
      key={i}
      col={i + props.colOffset}
      connectedRows={hyperEdge.indices}
      numbersOnRows={hyperEdge.frequencies}
      nameToShow={hyperEdge.nameToShow}
      rowHeight={props.rowHeight}
      xOffset={
        props.hyperedgeColumnWidth / 2 +
        i * props.hyperedgeColumnWidth +
        props.gapBetweenRanges
      }
      radius={8}
      onMouseEnter={props.onMouseEnter}
      onMouseLeave={props.onMouseLeave}
    />
  ));

  // * 3 because we have a gapBetweenRanges as padding
  const xOffset =
    props.xOffset +
    (props.index * 3 + 1) * props.gapBetweenRanges +
    props.colOffset * props.hyperedgeColumnWidth;

  return (
    <g transform={'translate(' + xOffset + ', +' + props.yOffset + ')'}>
      <text
        textDecoration={'underline'}
        x="10"
        y={props.rowHeight / 2}
        dy=".35em"
        style={{ fontWeight: 600, transform: 'translate3d(-10px, 15px, 0px) rotate(-30deg)' }}
      >
        {props.text}
      </text>
      <g transform={'translate(0,' + props.rowHeight + ')'}>
        {rows}
        {hyperedges}
      </g>
    </g>
  );
}

type HyperEdgeProps = {
  col: number;
  connectedRows: number[];
  numbersOnRows: number[];
  rowHeight: number;
  xOffset: number;
  radius: number;
  nameToShow: string;
  onMouseEnter: (col: number, nameToShow: string) => void;
  onMouseLeave: (col: number) => void;
};

const HyperEdge = (props: HyperEdgeProps) => {
  const ref = useRef<SVGGElement>(null);

  useEffect(() => {
    if (ref.current) {
      /** Adds mouse events when the user hovers over and out of a hyperedge. */
      select(ref.current)
        .on('mouseover', () => props.onMouseEnter(props.col, props.nameToShow))
        .on('mouseout', () => props.onMouseLeave(props.col));
    }
  }, [ref.current])


  // render all circles on the correct position.
  const circles = props.connectedRows.map((row, i) => {
    return (
      <g key={'row' + row + ' col' + i}>
        <circle
          cx={0}
          cy={row * props.rowHeight}
          r={props.radius}
          fill={'white'}
          stroke={'black'}
          strokeWidth={1}
        />
      </g>
    );
  });

  // create a line between two circles.
  const lines: JSX.Element[] = [];
  let y1 = props.connectedRows[0] * props.rowHeight + props.radius;
  let y2;
  for (let i = 1; i < props.connectedRows.length; i++) {
    y2 = props.connectedRows[i] * props.rowHeight - props.radius;
    lines.push(
      <line key={'line' + i} x1={0} y1={y1} x2={0} y2={y2} stroke={'black'} strokeWidth={1} />,
    );
    y1 = props.connectedRows[i] * props.rowHeight + props.radius;
  }

  const yOffset = props.rowHeight * 0.5;

  return (
    <g
      ref={ref}
      className={'col-' + props.col}
      transform={'translate(' + props.xOffset + ',' + yOffset + ')'}
    >
      {lines}
      {circles}
    </g>
  );
}
