/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { ChangeEventHandler, ReactElement, useState, MouseEventHandler, useEffect } from 'react';
import styles from './PaohvisFilterComponent.module.scss';
import { AttributeNames, FilterType } from '../Types';
import { useImmer } from 'use-immer';
import { Button, MenuItem, TextField } from '@mui/material';
import { useGraphQueryResult, useSchemaGraph } from '@graphpolaris/shared/lib/data-access';
import { isNodeLinkResult } from '../../shared/ResultNodeLinkParserUseCase';
import { isSchemaResult } from '../../shared/SchemaResultType';
import { calculateAttributesAndRelations, calculateAttributesFromRelation } from '../utils/utils';
import { boolPredicates, numberPredicates, textPredicates } from '../models/FilterPredicates';


type PaohvisFilterProps = {
  axis: FilterType;
  entityVertical: string;
  entityHorizontal: string;
  relationName: string;

  filterPaohvis(
    isTargetEntity: boolean,
    filterTarget: string,
    attributeName: string,
    predicate: string,
    compareValue: string,
  ): void;
  resetFilter(isTargetEntity: boolean): void;
};

type PaohvisFilterState = {
  filterTarget: string;
  attributeNameAndType: string;
  attributeType: string;
  predicate: string;
  compareValue: string;
  isFilterButtonEnabled: boolean;
  predicateTypeList: string[];
  attributeNamesOptions: string[];

  namesPerEntityOrRelation: string[];
  attributesPerEntityOrRelation: Record<string, AttributeNames>;

};

/** Component for rendering the filters for PAOHvis */
export const PaohvisFilterComponent = (props: PaohvisFilterProps) => {
  const graphQueryResult = useGraphQueryResult();
  const schema = useSchemaGraph();
  const isTargetEntity = props.axis != FilterType.edge;

  const [state, setState] = useImmer<PaohvisFilterState>({
    filterTarget: '',
    attributeNameAndType: '',
    attributeType: '',
    predicate: '',
    compareValue: '',
    isFilterButtonEnabled: false,
    predicateTypeList: [],
    attributeNamesOptions: [],
    namesPerEntityOrRelation: [],
    attributesPerEntityOrRelation: {},
  });

  /**
 * Updates the list of attributes that can be chosen from for the filter component.
 * Takes all attributes from the filter target and converts them to the format 'name:type'.
 */
  function updateAttributeNameOptions() {
    const filterTarget = state.filterTarget;
    const namesPerEntityOrRelation = state.namesPerEntityOrRelation;
    const attributesPerEntityOrRelation = state.attributesPerEntityOrRelation;

    if (!namesPerEntityOrRelation.includes(filterTarget))
      throw new Error('The filter target does not exist in the schema');

    setState(draft => {
      // Add all possible options for attributes to attributeNamesOptions
      // check all text attributes
      if (attributesPerEntityOrRelation[filterTarget].textAttributeNames.length > 0)
        attributesPerEntityOrRelation[filterTarget].textAttributeNames.map((attributeName) =>
          draft.attributeNamesOptions.push(`${attributeName}:text`),
        );

      // check all number attributes
      if (attributesPerEntityOrRelation[filterTarget].numberAttributeNames.length > 0)
        attributesPerEntityOrRelation[filterTarget].numberAttributeNames.map((attributeName) =>
          draft.attributeNamesOptions.push(`${attributeName}:number`),
        );

      // check all bool attributes
      if (attributesPerEntityOrRelation[filterTarget].boolAttributeNames.length > 0)
        attributesPerEntityOrRelation[filterTarget].boolAttributeNames.map((attributeName) =>
          draft.attributeNamesOptions.push(`${attributeName}:bool`),
        );
      return draft;
    });
  };

  /**
   * This is called when the name of the attribute is changed in a filter component.
   * Based on the type of the chosen attribute, a different list with predicates is generated.
   * @param event that called this eventhandler.
   */
  function onChangeAttributeName(event: React.ChangeEvent<HTMLInputElement>): void {
    const newAttributeNameAndType = event.target.value;
    const newAttributeSplit = event.target.value.split(':');
    const newAttributeType = newAttributeSplit[1];

    setState(draft => {
      if (!containsFilterTargetChosenAttribute(newAttributeNameAndType))
        throw new Error(
          'The chosen attribute does not exist in the entity/relation that will be filtered',
        );

      switch (newAttributeType) {
        case 'text':
          draft.predicateTypeList = Object.keys(textPredicates);
          break;
        case 'number':
          draft.predicateTypeList = Object.keys(numberPredicates);
          break;
        case 'bool':
          draft.predicateTypeList = Object.keys(boolPredicates);
          break;
      }

      draft.attributeNameAndType = newAttributeNameAndType;
      draft.attributeType = newAttributeType;
      draft.predicate = '';
      draft.compareValue = '';
      draft.isFilterButtonEnabled = false;
      return draft;
    });
  };

  /**
* This is called when the value of the predicate is changed in the filter component.
* @param event that called this eventhandler.
*/
  function onChangePredicate(event: React.ChangeEvent<HTMLInputElement>): void {
    const newpredicate = event.target.value;

    if (!isPredicateValid(newpredicate)) throw new Error('The chosen predicate is invalid');

    setState(draft => {
      draft.predicate = newpredicate;
      draft.compareValue = '';
      draft.isFilterButtonEnabled = false;
      return draft;
    });
  };

  /**
   * This is called when the value to compare the attribute with is changed in a filter component.
   * Based on the type of the chosen attribute, the compareValue has some conditions on what type it should be.
   * If attribute is numerical, the value has to be a number, etc.
   * @param event that called this eventhandler.
   */
  function onChangeCompareValue(event: React.ChangeEvent<HTMLInputElement>): void {
    setState(draft => {
      draft.compareValue = event.target.value;
      draft.isFilterButtonEnabled = isFilterConfigurationValid();
      return draft;
    });
  };

  /**
* This is called when the "apply filter" button is clicked.
* It checks if the input is correct and sends the information for the filters to the PaohvisViewModelImpl.
*/
  function onClickFilterPaohvisButton(): void {
    if (!isFilterConfigurationValid())
      throw new Error('Error: chosen attribute of predicate-value is invalid.');

    props.filterPaohvis(
      isTargetEntity,
      state.filterTarget,
      state.attributeNameAndType,
      state.predicate,
      state.compareValue,
    );
  };

  /**
 * This resets the filters for the chosen filter component.
 */
  function onClickResetFilter(): void {
    setState(draft => {
      draft.attributeNameAndType = '';
      draft.attributeType = '';
      draft.predicate = '';
      draft.compareValue = '';
      draft.isFilterButtonEnabled = false;
      draft.predicateTypeList = [];
      return draft;
    });
    props.resetFilter(isTargetEntity);
  };

  /**
 * Checks if the filter configuration is valid.
 * @returns {boolean} true if the filter configuration is valid.
 */
  function isFilterConfigurationValid(): boolean {
    return (
      containsFilterTargetChosenAttribute(state.attributeNameAndType) &&
      isPredicateValid(state.predicate) &&
      isCompareValueTypeValid(state.compareValue)
    );
  }
  /**
   * Checks if the given predicate is valid for the filter target.
   * @param predicate that should be checked.
   * @returns {boolean} true if the predicate is valid.
   */
  function isPredicateValid(predicate: string): boolean {
    return state.predicateTypeList.includes(predicate);
  }

  /**
   * This is used when the compareValue has changed.
   * @param {string} compareValue is the value that will be used with the predicate.
   * @returns {boolean} true if the chosen compareValue and chosen attribute are a valid combination.
   */
  function isCompareValueTypeValid(compareValue: string): boolean {
    const lowerCaseCompareValue: string = compareValue.toLowerCase();
    return (
      (state.attributeType == 'number' && !Number.isNaN(Number(compareValue))) ||
      (state.attributeType == 'bool' && (lowerCaseCompareValue == 'true' || lowerCaseCompareValue == 'false')) ||
      (state.attributeType == 'text' && compareValue.length > 0)
    );
  }

  /**
   * This determines whether the given attribute belongs to the filter target.
   * @param attributeNameAndType is the chosen attribute in the format of 'name:type'
   * @returns {boolean} true when the given attribute belongs to the filter target.
   */
  function containsFilterTargetChosenAttribute(attributeNameAndType: string): boolean {
    const filterTarget = state.filterTarget;
    const split: string[] = attributeNameAndType.split(':');
    const attributeName = split[0];
    const attributeType = split[1];
    const attributesPerEntityOrRelation = state.attributesPerEntityOrRelation;

    switch (attributeType) {
      case 'text':
        return attributesPerEntityOrRelation[filterTarget].textAttributeNames.includes(attributeName,);
      case 'number':
        return attributesPerEntityOrRelation[filterTarget].numberAttributeNames.includes(attributeName,);
      case 'bool':
        return attributesPerEntityOrRelation[filterTarget].boolAttributeNames.includes(attributeName,);
      default:
        return false;
    }
  }

  /**
   * This determines on which entity/relation the filter is going to be applied to.
   * @param entityVertical is the entity type that belongs to the Y-axis of the Paohvis table.
   * @param entityHorizontal is the entity type that belongs to the X-axis of the Paohvis table.
   * @param relationName is the relation type that is that is displayed in the Paohvis table
   */
  function determineFilterTarget(
    entityVertical: string,
    entityHorizontal: string,
    relationName: string,
  ): void {
    setState(draft => {
      if (props.axis === FilterType.yaxis)
        draft.filterTarget = entityVertical;
      else if (props.axis === FilterType.xaxis)
        draft.filterTarget = entityHorizontal;
      else draft.filterTarget = relationName;
      return draft;
    });
  }

  //
  // REACT
  //

  useEffect(() => {
    if (isNodeLinkResult(graphQueryResult)) {
      setState(draft => {
        draft.filterTarget = '';
        draft.attributeNamesOptions = [];
        return draft;
      })
    } else {
      console.error('Invalid query result!')
    }
  }, [graphQueryResult]);

  useEffect(() => {
    // if (isSchemaResult(schema)) {
    setState(draft => {
      if (isTargetEntity) {
        const entitiesFromSchema = calculateAttributesAndRelations(schema);
        draft.namesPerEntityOrRelation = entitiesFromSchema.entityNames;
        draft.attributesPerEntityOrRelation = entitiesFromSchema.attributesPerEntity;
      } else {
        const relationsFromSchema = calculateAttributesFromRelation(schema);
        draft.namesPerEntityOrRelation = relationsFromSchema.relationCollection;
        draft.attributesPerEntityOrRelation = relationsFromSchema.attributesPerRelation;
      }

      draft.filterTarget = '';
      draft.attributeNamesOptions = [];
      return draft;
    });
    // } else {
    //   console.error('Invalid schema!')
    // }
  }, [schema]);


  useEffect(() => {
    determineFilterTarget(props.entityVertical, props.entityHorizontal, props.relationName);
    if (state.filterTarget !== '')
      updateAttributeNameOptions();
  }, [props]);

  //
  // RENDER
  //

  // Check if the given entity or relation is in the queryResult
  let attributeNameMenuItems: ReactElement[] = [];
  if (state.attributeNamesOptions.length > 0) {
    attributeNameMenuItems = state.attributeNamesOptions.map((attributeNameAndType) => {
      const attributeName = attributeNameAndType.split(':')[0];
      return (
        <MenuItem key={attributeName} value={attributeNameAndType}>
          {attributeNameAndType}
        </MenuItem>
      );
    });
  } else
    attributeNameMenuItems = [
      <MenuItem key="placeholder" value="" disabled>
        First select an entity/relation with one or more attributes
      </MenuItem>,
    ];

  let predicateTypeList: ReactElement[] = [];
  if (state.predicateTypeList.length > 0) {
    predicateTypeList = state.predicateTypeList.map((predicate) => (
      <MenuItem key={predicate} value={predicate}>
        {predicate}
      </MenuItem>
    ));
  } else
    predicateTypeList = [
      <MenuItem key="placeholder" value="" disabled>
        First select an attribute
      </MenuItem>,
    ];

  return (
    <div>
      <div className={styles.container}>
        <p className={styles.title}>PAOHVis filters{props.axis}:</p>
        <div className={styles.selectContainer}>
          <p className={styles.subtitle}>{state.filterTarget}</p>
          <div style={{ padding: 10, display: 'block', width: '100%' }}>
            <TextField
              select
              id="standard-select-entity"
              style={{ minWidth: 120, marginRight: 20 }}
              label="Attribute"
              value={state.attributeNameAndType}
              onChange={onChangeAttributeName}
            >
              {attributeNameMenuItems}
            </TextField>
            <TextField
              select
              id="standard-select-relation"
              style={{ minWidth: 120, marginRight: 20 }}
              label="Relation"
              value={state.predicate}
              onChange={onChangePredicate}
            >
              {predicateTypeList}
            </TextField>
            <TextField
              id="standard-select-relation"
              style={{ minWidth: 120, marginRight: 20 }}
              label="Value"
              value={state.compareValue}
              onChange={onChangeCompareValue}
            >
              { }
            </TextField>

            <div style={{ height: 40, paddingTop: 10, marginBottom: 10 }}>
              <Button
                variant="contained"
                disabled={!state.isFilterButtonEnabled}
                onClick={onClickFilterPaohvisButton}
              >
                <span style={{ fontWeight: 'bold' }}>Apply filter</span>
              </Button>
            </div>
            <div style={{ height: 40, marginBottom: 20 }}>
              <Button
                variant="contained"
                onClick={onClickResetFilter}
              >
                <span style={{ fontWeight: 'bold' }}>Reset filter</span>
              </Button>
            </div>
          </div>
        </div>
      </div>
    </div>
  );
}
