import Broker from '../../../../domain/entity/broker/broker';
import SemanticSubstratesViewModelImpl from './SemanticSubstratesViewModelImpl';
import mockNodeLinkResult from '../../../../data/mock-data/query-result/big2ndChamberQueryResult';
import mockSchema from '../../../../data/mock-data/schema-result/2ndChamberSchemaMock';
import { MinMaxType } from '../../../../domain/entity/semantic-substrates/structures/Types';

jest.mock('../../../view/result-visualisations/semantic-substrates/SemanticSubstratesStylesheet');

describe('SemanticSubstratesViewModelImpl: Broker subscriptions', () => {
  it('should consume schema results when subscribed', () => {
    const viewModel = new SemanticSubstratesViewModelImpl();

    const mockConsumeMessages = jest.fn();
    viewModel.consumeMessageFromBackend = mockConsumeMessages;

    viewModel.subscribeToSchemaResult();
    Broker.instance().publish('test schema result', 'schema_result');
    expect(mockConsumeMessages.mock.calls[0][0]).toEqual('test schema result');

    viewModel.unSubscribeFromSchemaResult();
    Broker.instance().publish('test schema result', 'schema_result');
    expect(mockConsumeMessages).toBeCalledTimes(1);
  });
});

describe('SemanticSubstratesViewModelImpl:OnPlotTitleChanged', () => {
  it('should change the plot title', () => {
    // Initialize a ViewModel with some plots.
    const viewModelImpl = new SemanticSubstratesViewModelImpl();
    viewModelImpl.consumeMessageFromBackend(mockSchema);
    viewModelImpl.consumeMessageFromBackend(mockNodeLinkResult);

    const onViewModelChangedMock = jest.fn();
    const baseViewMock = {
      onViewModelChanged: onViewModelChangedMock,
    };
    viewModelImpl.attachView(baseViewMock);

    const i = 1;
    const oldPlotSpec = viewModelImpl.plotSpecifications[i];
    viewModelImpl.onPlotTitleChanged(i, 'commissies', 'naam', 'Defensie');

    expect(viewModelImpl.plotSpecifications[i]).toEqual({
      ...oldPlotSpec,
      entity: 'commissies',
      labelAttributeType: 'naam',
      labelAttributeValue: 'Defensie',
    });

    // And check if the NotifyViewAboutChanges is called.
    expect(onViewModelChangedMock).toBeCalledTimes(1);

    viewModelImpl.onPlotTitleChanged(i, 'commissies', 'id', 'Defensie');
    expect(viewModelImpl.plotSpecifications[i]).toEqual({
      ...oldPlotSpec,
      entity: 'commissies',
      labelAttributeType: 'id',
      labelAttributeValue: 'Defensie',
    });
  });
});

describe('SemanticSubstratesViewModelImpl', () => {
  it('should apply default specs on consuming a new node link result', () => {
    const viewModelImpl = new SemanticSubstratesViewModelImpl();

    const onViewModelChangedMock = jest.fn();
    const baseViewMock = {
      onViewModelChanged: onViewModelChangedMock,
    };
    viewModelImpl.attachView(baseViewMock);

    viewModelImpl.consumeMessageFromBackend(mockSchema);

    // Consume a nodelink query result.
    viewModelImpl.consumeMessageFromBackend(mockNodeLinkResult);

    let mockCalc = viewModelImpl.scaleCalculation;
    let mockCalcColour = viewModelImpl.colourCalculation;

    const expectedPlots = [
      {
        title: 'partij:VVD',
        nodes: [
          {
            id: 'kamerleden/148',
            data: { text: 'kamerleden/148' },
            originalPosition: { x: 1, y: 0 },
            attributes: {
              anc: 1526,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/9ab41898-332f-4c08-a834-d58b4669c990.jpg?itok=pbUPRriP',
              leeftijd: 43,
              naam: 'Dilan Yeilgz-Zegerius',
              partij: 'VVD',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 114.28571428571432, y: 100 },
          },
          {
            id: 'kamerleden/132',
            data: { text: 'kamerleden/132' },
            originalPosition: { x: 2, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/b625d2bf-da74-40a2-b013-2069a09dcb6c.jpg?itok=zMgno8bi',
              leeftijd: 36,
              naam: 'Peter Valstar',
              partij: 'VVD',
              woonplaats: "'s-Gravenzande",
            },
            scaledPosition: { x: 133.99014778325127, y: 100 },
          },
          {
            id: 'kamerleden/131',
            data: { text: 'kamerleden/131' },
            originalPosition: { x: 3, y: 0 },
            attributes: {
              anc: 1304,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/97ffe642-648f-4dab-a0e8-0720e96e3e35.jpg?itok=eDllv9dS',
              leeftijd: 49,
              naam: 'Judith Tielen',
              partij: 'VVD',
              woonplaats: 'Utrecht',
            },
            scaledPosition: { x: 153.6945812807882, y: 100 },
          },
          {
            id: 'kamerleden/128',
            data: { text: 'kamerleden/128' },
            originalPosition: { x: 4, y: 0 },
            attributes: {
              anc: 3171,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/2538ca6a-8254-4c7e-af71-a20db82afd30.jpg?itok=GvwO8NYO',
              leeftijd: 46,
              naam: 'Ockje Tellegen',
              partij: 'VVD',
              woonplaats: "'s-Gravenhage",
            },
            scaledPosition: { x: 173.39901477832515, y: 100 },
          },
          {
            id: 'kamerleden/117',
            data: { text: 'kamerleden/117' },
            originalPosition: { x: 5, y: 0 },
            attributes: {
              anc: 2004,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/8b3664bd-77e4-468b-af96-f3f4ec27fcce.jpg?itok=ofrc9cnP',
              leeftijd: 54,
              naam: 'Mark Rutte',
              partij: 'VVD',
              woonplaats: "'s-Gravenhage",
            },
            scaledPosition: { x: 193.1034482758621, y: 100 },
          },
          {
            id: 'kamerleden/106',
            data: { text: 'kamerleden/106' },
            originalPosition: { x: 6, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/0e34f732-337a-4339-961f-8c18d68e714d.jpg?itok=FTBMCT9a',
              leeftijd: 54,
              naam: 'Marille Paul',
              partij: 'VVD',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 212.80788177339906, y: 100 },
          },
          {
            id: 'kamerleden/100',
            data: { text: 'kamerleden/100' },
            originalPosition: { x: 7, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/2b223ce1-e251-462b-a5fc-971c92bbf37c.jpg?itok=MD8qI-l1',
              leeftijd: 43,
              naam: 'Daan de Neef',
              partij: 'VVD',
              woonplaats: 'Breda',
            },
            scaledPosition: { x: 232.512315270936, y: 100 },
          },
          {
            id: 'kamerleden/97',
            data: { text: 'kamerleden/97' },
            originalPosition: { x: 8, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/cd02edbc-106e-46d1-8d70-6594741356ea.jpg?itok=Q9q13ntf',
              leeftijd: 33,
              naam: 'Fahid Minhas',
              partij: 'VVD',
              woonplaats: 'Schiedam',
            },
            scaledPosition: { x: 252.21674876847294, y: 100 },
          },
          {
            id: 'kamerleden/81',
            data: { text: 'kamerleden/81' },
            originalPosition: { x: 9, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/8767e9de-869e-4e8a-940f-1a63d15eab3c.jpg?itok=tdoE8yQX',
              leeftijd: 28,
              naam: 'Daan de Kort',
              partij: 'VVD',
              woonplaats: 'Veldhoven',
            },
            scaledPosition: { x: 271.9211822660099, y: 100 },
          },
          {
            id: 'kamerleden/79',
            data: { text: 'kamerleden/79' },
            originalPosition: { x: 10, y: 0 },
            attributes: {
              anc: 1526,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/88e0734b-8c03-4daa-8ec5-055e36c07030.jpg?itok=_ZteUeWk',
              leeftijd: 39,
              naam: 'Daniel Koerhuis',
              partij: 'VVD',
              woonplaats: 'Raalte',
            },
            scaledPosition: { x: 291.6256157635468, y: 100 },
          },
          {
            id: 'kamerleden/145',
            data: { text: 'kamerleden/145' },
            originalPosition: { x: 11, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/8ee20f38-f3f0-4ff9-b5d9-84557b5ddcb1.jpg?itok=ExYCR1Ti',
              leeftijd: 51,
              naam: 'Hatte van der Woude',
              partij: 'VVD',
              woonplaats: 'Delft',
            },
            scaledPosition: { x: 311.3300492610838, y: 100 },
          },
          {
            id: 'kamerleden/70',
            data: { text: 'kamerleden/70' },
            originalPosition: { x: 12, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/f23be4bd-7b2b-45dd-be84-4f76f11b624a.jpg?itok=DsfJ-8o0',
              leeftijd: 43,
              naam: 'Roelien Kamminga',
              partij: 'VVD',
              woonplaats: 'Groningen',
            },
            scaledPosition: { x: 331.03448275862075, y: 100 },
          },
          {
            id: 'kamerleden/127',
            data: { text: 'kamerleden/127' },
            originalPosition: { x: 13, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/ce400601-651e-4ac9-a43e-ed8d4817b7fd.jpg?itok=wsISYKCx',
              leeftijd: 44,
              naam: 'Pim van Strien',
              partij: 'VVD',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 350.7389162561577, y: 100 },
          },
          {
            id: 'kamerleden/115',
            data: { text: 'kamerleden/115' },
            originalPosition: { x: 14, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/4bf69aa9-29a7-44db-b7ee-8bfa40e343dd.jpg?itok=-BKeAg8F',
              leeftijd: 32,
              naam: 'Queeny Rajkowski',
              partij: 'VVD',
              woonplaats: 'Utrecht',
            },
            scaledPosition: { x: 370.44334975369463, y: 100 },
          },
          {
            id: 'kamerleden/64',
            data: { text: 'kamerleden/64' },
            originalPosition: { x: 15, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/7954eb2c-1bf5-42af-80dc-85f29b367db0.jpg?itok=9ucjAERV',
              leeftijd: 49,
              naam: 'Folkert Idsinga',
              partij: 'VVD',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 390.1477832512316, y: 100 },
          },
          {
            id: 'kamerleden/60',
            data: { text: 'kamerleden/60' },
            originalPosition: { x: 16, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/8509e44d-0f9b-413a-a2cd-f64d656c3955.jpg?itok=Iy2_G4Jx',
              leeftijd: 53,
              naam: 'Jacqueline van den Hil',
              partij: 'VVD',
              woonplaats: 'Goes',
            },
            scaledPosition: { x: 409.8522167487685, y: 100 },
          },
          {
            id: 'kamerleden/143',
            data: { text: 'kamerleden/143' },
            originalPosition: { x: 17, y: 0 },
            attributes: {
              anc: 1823,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/7d18b6e3-de0c-4aea-bef8-242bbc38e936.jpg?itok=zebZoGjq',
              leeftijd: 43,
              naam: 'Jeroen van Wijngaarden',
              partij: 'VVD',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 429.55665024630554, y: 100 },
          },
          {
            id: 'kamerleden/56',
            data: { text: 'kamerleden/56' },
            originalPosition: { x: 18, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/a3b17348-54cf-4bca-82d4-e62bd46c2fbc.jpg?itok=MF1OOpeK',
              leeftijd: 40,
              naam: 'Eelco Heinen',
              partij: 'VVD',
              woonplaats: "'s-Gravenhage",
            },
            scaledPosition: { x: 449.26108374384245, y: 100 },
          },
          {
            id: 'kamerleden/142',
            data: { text: 'kamerleden/142' },
            originalPosition: { x: 19, y: 0 },
            attributes: {
              anc: 1526,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/c3b1dca8-2cb4-4025-9527-36163a098c1f.jpg?itok=3BnXwQgy',
              leeftijd: 35,
              naam: 'Dennis Wiersma',
              partij: 'VVD',
              woonplaats: 'De Bilt',
            },
            scaledPosition: { x: 468.96551724137936, y: 100 },
          },
          {
            id: 'kamerleden/46',
            data: { text: 'kamerleden/46' },
            originalPosition: { x: 20, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/a0b48551-42b7-4c55-9556-6b997d978ff2.jpg?itok=-A3mAzHe',
              leeftijd: 41,
              naam: 'Peter de Groot',
              partij: 'VVD',
              woonplaats: 'Harderwijk',
            },
            scaledPosition: { x: 488.6699507389164, y: 100 },
          },
          {
            id: 'kamerleden/38',
            data: { text: 'kamerleden/38' },
            originalPosition: { x: 21, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/9c8bc5bc-8d13-4c11-be37-57a43a1d734b.jpg?itok=TqEaAJCz',
              leeftijd: 30,
              naam: 'Silvio Erkens',
              partij: 'VVD',
              woonplaats: 'Kerkrade',
            },
            scaledPosition: { x: 508.3743842364533, y: 100 },
          },
          {
            id: 'kamerleden/35',
            data: { text: 'kamerleden/35' },
            originalPosition: { x: 22, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/ea21d788-7cfe-41ff-b22f-0664397139af.jpg?itok=4G8NOaak',
              leeftijd: 32,
              naam: 'Ulysse Ellian',
              partij: 'VVD',
              woonplaats: 'Almere',
            },
            scaledPosition: { x: 528.0788177339903, y: 100 },
          },
          {
            id: 'kamerleden/33',
            data: { text: 'kamerleden/33' },
            originalPosition: { x: 23, y: 0 },
            attributes: {
              anc: 1526,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/c4f7c468-721e-4953-97f2-bff05fe570c4.jpg?itok=MmXfOgJx',
              leeftijd: 41,
              naam: 'Zohair El Yassini',
              partij: 'VVD',
              woonplaats: 'Utrecht',
            },
            scaledPosition: { x: 547.7832512315272, y: 100 },
          },
          {
            id: 'kamerleden/25',
            data: { text: 'kamerleden/25' },
            originalPosition: { x: 24, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/bf494e35-c009-4c74-a5c6-3cb2cd26bb51.jpg?itok=2CTfrPhP',
              leeftijd: 31,
              naam: 'Thom van Campen',
              partij: 'VVD',
              woonplaats: 'Zwolle',
            },
            scaledPosition: { x: 567.4876847290642, y: 100 },
          },
          {
            id: 'kamerleden/54',
            data: { text: 'kamerleden/54' },
            originalPosition: { x: 25, y: 0 },
            attributes: {
              anc: 2601,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/522d59a0-10ba-488f-a27a-a1dda7581900.jpg?itok=5b_3G5nE',
              leeftijd: 43,
              naam: 'Rudmer Heerema',
              partij: 'VVD',
              woonplaats: 'Alkmaar',
            },
            scaledPosition: { x: 587.1921182266011, y: 100 },
          },
          {
            id: 'kamerleden/96',
            data: { text: 'kamerleden/96' },
            originalPosition: { x: 26, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/973da897-66c7-4010-a98d-505e0e97a60c.jpg?itok=HLNJtdsA',
              leeftijd: 44,
              naam: 'Ingrid Michon-Derkzen',
              partij: 'VVD',
              woonplaats: "'s-Gravenhage",
            },
            scaledPosition: { x: 606.896551724138, y: 100 },
          },
          {
            id: 'kamerleden/9',
            data: { text: 'kamerleden/9' },
            originalPosition: { x: 27, y: 0 },
            attributes: {
              anc: 1414,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/a962ffcb-a300-45e1-8850-70c7173c233e.jpg?itok=0f-c2lGz',
              leeftijd: 35,
              naam: 'Bente Becker',
              partij: 'VVD',
              woonplaats: "'s-Gravenhage",
            },
            scaledPosition: { x: 626.600985221675, y: 100 },
          },
          {
            id: 'kamerleden/23',
            data: { text: 'kamerleden/23' },
            originalPosition: { x: 28, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/e5c4753f-d2d8-4a4a-ae9d-62118892a731.jpg?itok=iGnPsRMX',
              leeftijd: 34,
              naam: 'Ruben Brekelmans',
              partij: 'VVD',
              woonplaats: 'Oisterwijk',
            },
            scaledPosition: { x: 646.3054187192118, y: 100 },
          },
          {
            id: 'kamerleden/135',
            data: { text: 'kamerleden/135' },
            originalPosition: { x: 29, y: 0 },
            attributes: {
              anc: 3122,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/b8d002bc-ea6c-4f57-b36e-38415b205e09.jpg?itok=tHQU-Qrn',
              leeftijd: 56,
              naam: 'Aukje de Vries',
              partij: 'VVD',
              woonplaats: 'Leeuwarden',
            },
            scaledPosition: { x: 666.0098522167489, y: 100 },
          },
          {
            id: 'kamerleden/0',
            data: { text: 'kamerleden/0' },
            originalPosition: { x: 30, y: 0 },
            attributes: {
              anc: 987,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/397c857a-fda0-414d-8fdc-8288cd3284aa.jpg?itok=55l5zRvr',
              leeftijd: 31,
              naam: 'Thierry Aartsen',
              partij: 'VVD',
              woonplaats: 'Breda',
            },
            scaledPosition: { x: 685.7142857142858, y: 100 },
          },
        ],
        selectedAttributeCatecorigal: 'state',
        selectedAttributeNumerical: 'long',
        scaleCalculation: mockCalc,
        colourCalculation: mockCalcColour,
        minmaxXAxis: { min: -4.800000000000001, max: 35.8 },
        minmaxYAxis: { min: -1, max: 1 },
        width: 800,
        height: 200,
        yOffset: 0,
        possibleTitleAttributeValues: [
          'VVD',
          'D66',
          'GL',
          'PVV',
          'PvdD',
          'PvdA',
          'SGP',
          'BIJ1',
          'CU',
          'BBB',
          'CDA',
          'SP',
          'FVD',
          'DENK',
          'Fractie Den Haan',
          'Volt',
          'JA21',
        ],
      },
      {
        title: 'partij:D66',
        nodes: [
          {
            id: 'kamerleden/141',
            data: { text: 'kamerleden/141' },
            originalPosition: { x: 1, y: 0 },
            attributes: {
              anc: 3171,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/d1bb566e-a111-43b7-8856-273ebac1f753.jpg?itok=-k-RCynx',
              leeftijd: 48,
              naam: 'Steven van Weyenberg',
              partij: 'D66',
              woonplaats: "'s-Gravenhage",
            },
            scaledPosition: { x: 114.2857142857143, y: 100 },
          },
          {
            id: 'kamerleden/138',
            data: { text: 'kamerleden/138' },
            originalPosition: { x: 2, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/6a54c095-9872-4322-81be-114b74233d40.jpg?itok=uf9jc1Gk',
              leeftijd: 36,
              naam: 'Hanneke van der Werf',
              partij: 'D66',
              woonplaats: "'s-Gravenhage",
            },
            scaledPosition: { x: 140.25974025974028, y: 100 },
          },
          {
            id: 'kamerleden/123',
            data: { text: 'kamerleden/123' },
            originalPosition: { x: 3, y: 0 },
            attributes: {
              anc: 1304,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/da9f33b0-80ca-49eb-a1ec-def3b46538d9.jpg?itok=nL6HIBWH',
              leeftijd: 38,
              naam: 'Joost Sneller',
              partij: 'D66',
              woonplaats: "'s-Gravenhage",
            },
            scaledPosition: { x: 166.23376623376626, y: 100 },
          },
          {
            id: 'kamerleden/114',
            data: { text: 'kamerleden/114' },
            originalPosition: { x: 4, y: 0 },
            attributes: {
              anc: 1414,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/6fc5878e-d76a-4e9d-a4e3-e5ef581bb2ff.jpg?itok=qf_oudwu',
              leeftijd: 30,
              naam: 'Rens Raemakers',
              partij: 'D66',
              woonplaats: 'Neer',
            },
            scaledPosition: { x: 192.20779220779224, y: 100 },
          },
          {
            id: 'kamerleden/147',
            data: { text: 'kamerleden/147' },
            originalPosition: { x: 5, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/45ef5aae-a6d0-4bad-861e-dd5b883a9665.jpg?itok=h5aEZJVM',
              leeftijd: 56,
              naam: 'Jorien Wuite',
              partij: 'D66',
              woonplaats: 'Voorburg',
            },
            scaledPosition: { x: 218.18181818181822, y: 100 },
          },
          {
            id: 'kamerleden/107',
            data: { text: 'kamerleden/107' },
            originalPosition: { x: 6, y: 0 },
            attributes: {
              anc: 42,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/d89044ef-b7df-4a15-be10-d342fe381664.jpg?itok=gmiV1adi',
              leeftijd: 42,
              naam: 'Wieke Paulusma',
              partij: 'D66',
              woonplaats: 'Groningen',
            },
            scaledPosition: { x: 244.1558441558442, y: 100 },
          },
          {
            id: 'kamerleden/105',
            data: { text: 'kamerleden/105' },
            originalPosition: { x: 7, y: 0 },
            attributes: {
              anc: 1526,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/2690bccc-5463-459a-b1df-cb93e222b348.jpg?itok=UZOQRvuv',
              leeftijd: 37,
              naam: 'Jan Paternotte',
              partij: 'D66',
              woonplaats: 'Leiden',
            },
            scaledPosition: { x: 270.1298701298702, y: 100 },
          },
          {
            id: 'kamerleden/94',
            data: { text: 'kamerleden/94' },
            originalPosition: { x: 8, y: 0 },
            attributes: {
              anc: 3171,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/17f26a48-9f54-42a1-9890-c6da761a9a2f.jpg?itok=BA8JoO5G',
              leeftijd: 65,
              naam: 'Paul van Meenen',
              partij: 'D66',
              woonplaats: 'Leiden',
            },
            scaledPosition: { x: 296.10389610389615, y: 100 },
          },
          {
            id: 'kamerleden/86',
            data: { text: 'kamerleden/86' },
            originalPosition: { x: 9, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/fb291eb2-962a-482f-971a-0b62673f9a86.jpg?itok=2J7OszgV',
              leeftijd: 41,
              naam: 'Jeanet van der Laan',
              partij: 'D66',
              woonplaats: 'Lisse',
            },
            scaledPosition: { x: 322.0779220779221, y: 100 },
          },
          {
            id: 'kamerleden/71',
            data: { text: 'kamerleden/71' },
            originalPosition: { x: 10, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/8349fff0-99b3-449a-bd4d-932ae95b29da.jpg?itok=xYICmuaX',
              leeftijd: 37,
              naam: 'Hlya Kat',
              partij: 'D66',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 348.0519480519481, y: 100 },
          },
          {
            id: 'kamerleden/69',
            data: { text: 'kamerleden/69' },
            originalPosition: { x: 11, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/64457f28-37b6-4948-860a-f28bc62b99ae.jpg?itok=Q2hiR0vj',
              leeftijd: 59,
              naam: 'Sigrid Kaag',
              partij: 'D66',
              woonplaats: "'s-Gravenhage",
            },
            scaledPosition: { x: 374.02597402597405, y: 100 },
          },
          {
            id: 'kamerleden/68',
            data: { text: 'kamerleden/68' },
            originalPosition: { x: 12, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/666f31ca-041f-466d-9e78-804317921d69.jpg?itok=G3ozh6C3',
              leeftijd: 37,
              naam: 'Romke de Jong',
              partij: 'D66',
              woonplaats: 'Gorredijk',
            },
            scaledPosition: { x: 400.0000000000001, y: 100 },
          },
          {
            id: 'kamerleden/66',
            data: { text: 'kamerleden/66' },
            originalPosition: { x: 13, y: 0 },
            attributes: {
              anc: 1527,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/49be3576-cea3-46c0-87eb-89beb108248d.jpg?itok=VfqikY8P',
              leeftijd: 34,
              naam: 'Rob Jetten',
              partij: 'D66',
              woonplaats: 'Ubbergen',
            },
            scaledPosition: { x: 425.97402597402595, y: 100 },
          },
          {
            id: 'kamerleden/134',
            data: { text: 'kamerleden/134' },
            originalPosition: { x: 14, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/19c8d0fb-66ca-4b45-be82-057cb36e1e61.jpg?itok=Bkq8gaTU',
              leeftijd: 57,
              naam: 'Hans Vijlbrief',
              partij: 'D66',
              woonplaats: 'Woubrugge',
            },
            scaledPosition: { x: 451.94805194805195, y: 100 },
          },
          {
            id: 'kamerleden/52',
            data: { text: 'kamerleden/52' },
            originalPosition: { x: 15, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/03d66e4a-8698-44fb-9c29-625630da268c.jpg?itok=hZ9q6Llc',
              leeftijd: 39,
              naam: 'Alexander Hammelburg',
              partij: 'D66',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 477.9220779220779, y: 100 },
          },
          {
            id: 'kamerleden/51',
            data: { text: 'kamerleden/51' },
            originalPosition: { x: 16, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/425ddb12-0b71-44b6-9875-70b075dc99f7.jpg?itok=txVxBI0l',
              leeftijd: 34,
              naam: 'Kiki Hagen',
              partij: 'D66',
              woonplaats: 'Mijdrecht',
            },
            scaledPosition: { x: 503.8961038961039, y: 100 },
          },
          {
            id: 'kamerleden/47',
            data: { text: 'kamerleden/47' },
            originalPosition: { x: 17, y: 0 },
            attributes: {
              anc: 1525,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/762e0f40-f100-4896-85a2-b1d8818fae41.jpg?itok=YM5w7UBV',
              leeftijd: 53,
              naam: 'Tjeerd de Groot',
              partij: 'D66',
              woonplaats: 'Haarlem',
            },
            scaledPosition: { x: 529.8701298701299, y: 100 },
          },
          {
            id: 'kamerleden/121',
            data: { text: 'kamerleden/121' },
            originalPosition: { x: 18, y: 0 },
            attributes: {
              anc: 3171,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/bf030048-0434-42f1-bbed-8b0c04014db4.jpg?itok=G35Rmw4M',
              leeftijd: 39,
              naam: 'Sjoerd Sjoerdsma',
              partij: 'D66',
              woonplaats: "'s-Gravenhage",
            },
            scaledPosition: { x: 555.8441558441559, y: 100 },
          },
          {
            id: 'kamerleden/42',
            data: { text: 'kamerleden/42' },
            originalPosition: { x: 19, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/49e7fc50-90f5-4476-9275-c31f88864082.jpg?itok=n7ZLWx8e',
              leeftijd: 48,
              naam: 'Lisa van Ginneken',
              partij: 'D66',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 581.8181818181819, y: 100 },
          },
          {
            id: 'kamerleden/22',
            data: { text: 'kamerleden/22' },
            originalPosition: { x: 20, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/e85125f3-b7cf-426c-9355-412838893f55.jpg?itok=Qs1_f_yV',
              leeftijd: 42,
              naam: 'Faissal Boulakjar',
              partij: 'D66',
              woonplaats: 'Teteringen',
            },
            scaledPosition: { x: 607.7922077922078, y: 100 },
          },
          {
            id: 'kamerleden/21',
            data: { text: 'kamerleden/21' },
            originalPosition: { x: 21, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/13e26587-88d8-440b-bb93-527a8e845342.jpg?itok=18OPNvjY',
              leeftijd: 45,
              naam: 'Raoul Boucke',
              partij: 'D66',
              woonplaats: 'Rotterdam',
            },
            scaledPosition: { x: 633.7662337662338, y: 100 },
          },
          {
            id: 'kamerleden/14',
            data: { text: 'kamerleden/14' },
            originalPosition: { x: 22, y: 0 },
            attributes: {
              anc: 3171,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/aa0272f6-6e3c-41f7-b040-b774b30d2196.jpg?itok=IQu5H4Bx',
              leeftijd: 49,
              naam: 'Vera Bergkamp',
              partij: 'D66',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 659.7402597402597, y: 100 },
          },
          {
            id: 'kamerleden/12',
            data: { text: 'kamerleden/12' },
            originalPosition: { x: 23, y: 0 },
            attributes: {
              anc: 1948,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/29124ef6-e1d5-4138-bf01-8433afde7269.jpg?itok=qw0TE4jl',
              leeftijd: 42,
              naam: 'Salima Belhaj',
              partij: 'D66',
              woonplaats: 'Rotterdam',
            },
            scaledPosition: { x: 685.7142857142858, y: 100 },
          },
        ],
        selectedAttributeCatecorigal: 'state',
        selectedAttributeNumerical: 'long',
        scaleCalculation: mockCalc,
        colourCalculation: mockCalcColour,
        minmaxXAxis: { min: -3.4000000000000004, max: 27.4 },
        minmaxYAxis: { min: -1, max: 1 },
        width: 800,
        height: 200,
        yOffset: 250,
        possibleTitleAttributeValues: [
          'VVD',
          'D66',
          'GL',
          'PVV',
          'PvdD',
          'PvdA',
          'SGP',
          'BIJ1',
          'CU',
          'BBB',
          'CDA',
          'SP',
          'FVD',
          'DENK',
          'Fractie Den Haan',
          'Volt',
          'JA21',
        ],
      },
      {
        title: 'partij:GL',
        nodes: [
          {
            id: 'kamerleden/140',
            data: { text: 'kamerleden/140' },
            originalPosition: { x: 1, y: 0 },
            attributes: {
              anc: 1526,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/c7822b58-103f-4612-87ef-648be97192c6.jpg?itok=jDwKCC26',
              leeftijd: 39,
              naam: 'Lisa Westerveld',
              partij: 'GL',
              woonplaats: 'Nijmegen',
            },
            scaledPosition: { x: 114.28571428571432, y: 100 },
          },
          {
            id: 'kamerleden/89',
            data: { text: 'kamerleden/89' },
            originalPosition: { x: 2, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/66b2be3d-10f8-46a4-90c6-b86005df0a50.jpg?itok=obcIgEe-',
              leeftijd: 31,
              naam: 'Senna Maatoug',
              partij: 'GL',
              woonplaats: 'Leiden',
            },
            scaledPosition: { x: 209.52380952380958, y: 100 },
          },
          {
            id: 'kamerleden/124',
            data: { text: 'kamerleden/124' },
            originalPosition: { x: 3, y: 0 },
            attributes: {
              anc: 1526,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/b50b3f0b-afc8-46e5-bfda-cb60e2935a5a.jpg?itok=7R0cTE10',
              leeftijd: 55,
              naam: 'Bart Snels',
              partij: 'GL',
              woonplaats: 'Utrecht',
            },
            scaledPosition: { x: 304.7619047619048, y: 100 },
          },
          {
            id: 'kamerleden/87',
            data: { text: 'kamerleden/87' },
            originalPosition: { x: 4, y: 0 },
            attributes: {
              anc: 1526,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/75a44c3a-5914-4c8e-9ca4-d987853f5844.jpg?itok=kA8ZaoNf',
              leeftijd: 56,
              naam: 'Tom van der Lee',
              partij: 'GL',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 400.0000000000001, y: 100 },
          },
          {
            id: 'kamerleden/24',
            data: { text: 'kamerleden/24' },
            originalPosition: { x: 5, y: 0 },
            attributes: {
              anc: 1085,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/ccea26dd-dcae-4174-8945-ec5cb568e951.jpg?itok=z7gsBQYd',
              leeftijd: 51,
              naam: 'Laura Bromet',
              partij: 'GL',
              woonplaats: 'Monnickendam',
            },
            scaledPosition: { x: 495.23809523809535, y: 100 },
          },
          {
            id: 'kamerleden/20',
            data: { text: 'kamerleden/20' },
            originalPosition: { x: 6, y: 0 },
            attributes: {
              anc: 57,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/dab4cf29-4843-4261-b09b-973fe98dbf65.jpg?itok=szVeNy9I',
              leeftijd: 27,
              naam: 'Kauthar Bouchallikh',
              partij: 'GL',
              woonplaats: 'Amsterdam',
            },
            scaledPosition: { x: 590.4761904761906, y: 100 },
          },
          {
            id: 'kamerleden/34',
            data: { text: 'kamerleden/34' },
            originalPosition: { x: 7, y: 0 },
            attributes: {
              anc: 1637,
              img: 'https://www.tweedekamer.nl/sites/default/files/styles/member_parlement_profile_square/public/551f9a19-738c-4298-afe1-5ca7959ab74b.jpg?itok=24RXpYrd',
              leeftijd: 45,
              naam: 'Corinne Ellemeet',
              partij: 'GL',
              woonplaats: 'Abcoude',
            },
            scaledPosition: { x: 685.7142857142859, y: 100 },
          },
        ],
        selectedAttributeCatecorigal: 'state',
        selectedAttributeNumerical: 'long',
        scaleCalculation: mockCalc,
        colourCalculation: mockCalcColour,
        minmaxXAxis: { min: -0.20000000000000018, max: 8.2 },
        minmaxYAxis: { min: -1, max: 1 },
        width: 800,
        height: 200,
        yOffset: 500,
        possibleTitleAttributeValues: [
          'VVD',
          'D66',
          'GL',
          'PVV',
          'PvdD',
          'PvdA',
          'SGP',
          'BIJ1',
          'CU',
          'BBB',
          'CDA',
          'SP',
          'FVD',
          'DENK',
          'Fractie Den Haan',
          'Volt',
          'JA21',
        ],
      },
    ];

    // Check the plots and the relations.
    expect(viewModelImpl.plots).toEqual(expectedPlots);
    expect(viewModelImpl.allRelations).toEqual(expectedRelations);

    // Also check if notifyViewAboutChanges is called.
    expect(onViewModelChangedMock).toHaveBeenCalledTimes(2);
  });

  it('add a plot and delete itonDelete', () => {
    const viewModelImpl = new SemanticSubstratesViewModelImpl();

    const onViewModelChangedMock = jest.fn();
    const baseViewMock = {
      onViewModelChanged: onViewModelChangedMock,
    };
    viewModelImpl.attachView(baseViewMock);

    viewModelImpl.onDelete(1);
  });

  it('onBrush', () => {
    const viewModelImpl = new SemanticSubstratesViewModelImpl();
    viewModelImpl.consumeMessageFromBackend(mockSchema);
    viewModelImpl.consumeMessageFromBackend(mockNodeLinkResult);

    const onViewModelChangedMock = jest.fn();
    const baseViewMock = {
      onViewModelChanged: onViewModelChangedMock,
    };
    viewModelImpl.attachView(baseViewMock);

    let expectedFilters: { x: MinMaxType; y: MinMaxType } = {
      x: { min: 30, max: 40 },
      y: { min: 30, max: 40 },
    };

    viewModelImpl.onBrush(1, { min: 30, max: 40 }, { min: 30, max: 40 });

    expect(viewModelImpl.filtersPerPlot[1]).toEqual(expectedFilters);
  });

  it('onAxisChanges', () => {
    const viewModelImpl = new SemanticSubstratesViewModelImpl();
    viewModelImpl.consumeMessageFromBackend(mockSchema);
    viewModelImpl.consumeMessageFromBackend(mockNodeLinkResult);

    const onViewModelChangedMock = jest.fn();
    const baseViewMock = {
      onViewModelChanged: onViewModelChangedMock,
    };
    viewModelImpl.attachView(baseViewMock);

    const expectedX = 'nieuweXaxis';
    const expectedY = 'nieuweYaxis';
    const expectedX2 = 'evenly spaced';
    const expectedY2 = '# outbound connections';

    // change labels
    viewModelImpl.onAxisLabelChanged(1, 'x', 'nieuweXaxis');
    viewModelImpl.onAxisLabelChanged(1, 'y', 'nieuweYaxis');

    expect(viewModelImpl.plotSpecifications[1].xAxisAttributeType).toEqual(expectedX);
    expect(viewModelImpl.plotSpecifications[1].yAxisAttributeType).toEqual(expectedY);

    // change labels again
    viewModelImpl.onAxisLabelChanged(1, 'x', 'evenly spaced');
    viewModelImpl.onAxisLabelChanged(1, 'y', '# outbound connections');

    expect(viewModelImpl.plotSpecifications[1].xAxis).toEqual(expectedX2);
    expect(viewModelImpl.plotSpecifications[1].yAxis).toEqual(expectedY2);
  });

  it('onCheckboxChanges', () => {
    const viewModelImpl = new SemanticSubstratesViewModelImpl();
    viewModelImpl.consumeMessageFromBackend(mockSchema);
    viewModelImpl.consumeMessageFromBackend(mockNodeLinkResult);

    const onViewModelChangedMock = jest.fn();
    const baseViewMock = {
      onViewModelChanged: onViewModelChangedMock,
    };
    viewModelImpl.attachView(baseViewMock);

    viewModelImpl.onCheckboxChanged(1, 2, true);

    expect(viewModelImpl.visibleRelations[1][2]).toBeTruthy();
  });

  it('addPlot', () => {
    const viewModelImpl = new SemanticSubstratesViewModelImpl();
    viewModelImpl.consumeMessageFromBackend(mockSchema);
    viewModelImpl.consumeMessageFromBackend(mockNodeLinkResult);

    const onViewModelChangedMock = jest.fn();
    const baseViewMock = {
      onViewModelChanged: onViewModelChangedMock,
    };
    viewModelImpl.attachView(baseViewMock);

    expect(viewModelImpl.plots).toHaveLength(3);

    viewModelImpl.addPlot('partij', 'naam', 'VVD');

    expect(viewModelImpl.plots).toHaveLength(4);
  });
});

const deletePlot = [
  {
    title: 'partij:VVD',
    nodes: [
      {
        id: 'kamerleden/148',
        data: {
          text: 'kamerleden/148',
        },
        originalPosition: {
          x: 1,
          y: 0,
        },
        scaledPosition: {
          x: 114.28571428571432,
          y: 100,
        },
      },
    ],
  },
];

const expectedRelations = [
  [[], [], []],
  [[], [], []],
  [[], [], []],
];
